/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.visualization.views.voxelfilter;

import fr.amap.commons.util.swing.FrameView;
import fr.amap.commons.util.swing.JVerticalPanel;
import fr.amap.lidar.amapvox.nephos3d.entities.util.VoxelFilter;
import fr.amap.lidar.amapvox.nephos3d.entities.util.VoxelFilteringType;
import fr.amap.lidar.amapvox.nephos3d.entities.util.VoxelSpaceFile;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JSlider;
import javax.swing.border.LineBorder;
import org.jdesktop.swingx.JXButton;
import org.jdesktop.swingx.JXComboBox;
import org.jdesktop.swingx.JXLabel;
import org.jdesktop.swingx.JXPanel;
import org.jdesktop.swingx.JXTextField;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * FrameView of a voxel space filter.
 */
public class JXVoxelFilter extends FrameView {

    private List<JXVoxelFilterListener> listeners;

    private VoxelSpaceFile voxelFile;

    private final int index;

    private JXPanel contentPanel;
    
    private JVerticalPanel verticalPanel;

    private JXLabel propertyLabel;
    private JXComboBox propertyComboBox;

    private JXLabel filteringLabel;
    private JXComboBox filteringComboBox;
    private JCheckBox filteringNaNCheckBox;

    private JXPanel valuePanel;
    private JXLabel valueLabel;
    private JSlider valueSlider;
    private JXTextField valueTextField;

    private JXButton closeButton;

    public JXVoxelFilter() throws Exception {
        super();
        this.listeners = new ArrayList();
        final Random random = new Random();
        this.index = random.nextInt();
    }

    public final void addListener(final JXVoxelFilterListener listener) {
        listeners.add(listener);
    }

    public final void updatePropertyNames(final String[] names) {
        final ComboBoxModel model = new DefaultComboBoxModel(names);
        propertyComboBox.setModel(model);
        propertyComboBox.setSelectedItem(0);
    }

    public final void setDefaultProperty(final String defaultProperty) {
        propertyComboBox.setSelectedItem(defaultProperty);
    }

    public final void setVoxelFile(final VoxelSpaceFile voxelFile) {
        this.voxelFile = voxelFile;
        valueSlider.setMaximum(voxelFile.getDifferentValueQuantity(
                (String) propertyComboBox.getSelectedItem()) - 1);
    }

    public final JXPanel getContentPanel() {
        return contentPanel;
    }

    @Override
    protected void initContainer() throws Exception {
        getContentPane().setLayout(new BorderLayout());
    }

    @Override
    protected void initComponents() throws Exception {
        /* Init content panel */
        contentPanel = new JXPanel(new BorderLayout());
        contentPanel.setBorder(new LineBorder(Color.BLACK));
        contentPanel.setMaximumSize(new Dimension(10000, 100));
        getContentPane().add(contentPanel, BorderLayout.CENTER);

        verticalPanel = new JVerticalPanel();
        contentPanel.add(verticalPanel, BorderLayout.CENTER);
        

        /* Init property label */
        propertyLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            propertyLabel.setText(bundle.getString("voxelFilter_propertyLabel") + " :");
        });

        /* Init property combobox */
        propertyComboBox = new JXComboBox();
        addLocaleCallback((locale) -> {
            propertyComboBox.setToolTipText(bundle.getString("voxelFilter_propertyComboBox_tooltip"));
        });
        propertyComboBox.addItemListener((e) -> {
            valueSlider.setValue(0);
            updateFilter();
        });
        

        /* Init close button */
        closeButton = new JXButton(loadIcon("/icons/delete.png"));
        closeButton.setPreferredSize(new Dimension(26, 26));
        closeButton.addActionListener((e) -> {
            for (JXVoxelFilterListener listener : listeners) {
                listener.onClose(getFilter());
            }
        });
        
        verticalPanel.addLine(propertyLabel, propertyComboBox, closeButton);

        /*Init filtering label */
        filteringLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            filteringLabel.setText(bundle.getString("voxelFilter_filteringLabel") + " :");
        });

        /* Init filtering combobox */
        filteringComboBox = new JXComboBox();
        filteringComboBox.addItemListener((e) -> {
            updateFilter();
        });
        addLocaleCallback((locale) -> {
            final ComboBoxModel model = new DefaultComboBoxModel(new String[]{
                bundle.getString("voxelFilter_filteringComboBox_equal"),
                bundle.getString("voxelFilter_filteringComboBox_lesser"),
                bundle.getString("voxelFilter_filteringComboBox_greater"),
                bundle.getString("voxelFilter_filteringComboBox_notequal"),});
            filteringComboBox.setModel(model);
            filteringComboBox.setToolTipText(bundle.getString("voxelFilter_filteringComboBox_tooltip"));
        });

        filteringNaNCheckBox = new JCheckBox("NaN");
        addLocaleCallback((locale) -> {
            filteringNaNCheckBox.setToolTipText(bundle.getString("voxelFilter_filteringNaNCheckBox_tooltip"));
        });
        filteringNaNCheckBox.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
        filteringNaNCheckBox.addActionListener((e) -> {
            updateFilter();
        });
        
        verticalPanel.addLine(filteringLabel, filteringComboBox, filteringNaNCheckBox);

        /* Init value label */
        valueLabel = new JXLabel();
        addLocaleCallback((locale) -> {
            valueLabel.setText(bundle.getString("voxelFilter_valueLabel") + " :");
        });

        /* Init value slider */
        valueSlider = new JSlider(0, 100, 0);
        valueSlider.setPreferredSize(new Dimension(125, 20));
        addLocaleCallback((locale) -> {
            valueSlider.setToolTipText(bundle.getString("voxelFilter_valueSlider_tooltip"));
        });
        valueSlider.addChangeListener((e) -> {
            if (voxelFile != null) {
                final ExecutorService executor = Executors.newCachedThreadPool();
                executor.execute(() -> {
                    final String property = (String) propertyComboBox.getSelectedItem();
                    valueTextField.setText(Float.toString(
                            voxelFile.getSingleProperty(property, valueSlider.getValue())));
                });
                executor.shutdown();
                
                updateFilter();
            }
        });

        /* Init value text field */
        valueTextField = new JXTextField();
        valueTextField.setText("0.0");
        valueTextField.setEditable(false);
        valueTextField.setPreferredSize(new Dimension(100, valueTextField.getPreferredSize().height));

        verticalPanel.addLine(valueLabel, valueSlider, valueTextField);
    }

    private void updateFilter() {
        for (JXVoxelFilterListener listener : listeners) {
            listener.onChange(getFilter());
        }
    }

    public final VoxelFilter getFilter() {
        if (voxelFile != null) {
            final String property = (String) propertyComboBox.getSelectedItem();
            final float min = voxelFile.getMinPropertyValue(property);
            final float max = voxelFile.getMaxPropertyValue(property);
            final float value = voxelFile.getSingleProperty(property, valueSlider.getValue());
            final float normalizedValue = (value - min) / (max - min);
            
            return new VoxelFilter(
                    index,
                    property,
                    normalizedValue,
                    VoxelFilteringType.values()[filteringComboBox.getSelectedIndex()],
                    filteringNaNCheckBox.isSelected()
            );
        }
        
        return null;
    }

}
